@@ -157,7 +157,7 @@ DNSConnection::connect(sockaddr const *addr, Options const &opt)
       p                               = static_cast<uint16_t>((p % (LAST_RANDOM_PORT - FIRST_RANDOM_PORT)) + FIRST_RANDOM_PORT);
       ats_ip_port_cast(&bind_addr.sa) = htons(p); // stuff port in sockaddr.
       Debug("dns", "random port = %s", ats_ip_nptop(&bind_addr.sa, b, sizeof b));
-      if ((res = socketManager.ink_bind(fd, &bind_addr.sa, bind_size, Proto)) < 0) {
+      if (socketManager.ink_bind(fd, &bind_addr.sa, bind_size, Proto) < 0) {
         continue;
       }
       goto Lok;
