@@ -74,15 +74,12 @@
 #include "PluginVC.h"
 #include "P_EventSystem.h"
 #include "P_Net.h"
-#include "ts/Regression.h"
+#include "tscore/Regression.h"
 
 #define PVC_LOCK_RETRY_TIME HRTIME_MSECONDS(10)
 #define PVC_DEFAULT_MAX_BYTES 32768
 #define MIN_BLOCK_TRANSFER_BYTES 128
 
-#define EVENT_PTR_LOCKED (void *)0x1
-#define EVENT_PTR_CLOSED (void *)0x2
-
 #define PVC_TYPE ((vc_type == PLUGIN_VC_ACTIVE) ? "Active" : "Passive")
 
 PluginVC::PluginVC(PluginVCCore *core_obj)
@@ -211,9 +208,6 @@ PluginVC::main_handler(int event, void *data)
   } else if (call_event == inactive_event) {
     if (inactive_timeout_at && inactive_timeout_at < Thread::get_hrtime()) {
       process_timeout(&inactive_event, VC_EVENT_INACTIVITY_TIMEOUT);
-      if (nullptr == inactive_event) {
-        call_event->cancel();
-      }
     }
   } else {
     if (call_event == sm_lock_retry_event) {
@@ -263,7 +257,7 @@ PluginVC::do_io_read(Continuation *c, int64_t nbytes, MIOBuffer *buf)
   // Note: we set vio.op last because process_read_side looks at it to
   //  tell if the VConnection is active.
   read_state.vio.mutex     = c ? c->mutex : this->mutex;
-  read_state.vio._cont     = c;
+  read_state.vio.cont      = c;
   read_state.vio.nbytes    = nbytes;
   read_state.vio.ndone     = 0;
   read_state.vio.vc_server = (VConnection *)this;
@@ -295,7 +289,7 @@ PluginVC::do_io_write(Continuation *c, int64_t nbytes, IOBufferReader *abuffer,
   // Note: we set vio.op last because process_write_side looks at it to
   //  tell if the VConnection is active.
   write_state.vio.mutex     = c ? c->mutex : this->mutex;
-  write_state.vio._cont     = c;
+  write_state.vio.cont      = c;
   write_state.vio.nbytes    = nbytes;
   write_state.vio.ndone     = 0;
   write_state.vio.vc_server = (VConnection *)this;
@@ -512,7 +506,7 @@ PluginVC::process_write_side(bool other_side_call)
   Debug("pvc", "[%u] %s: process_write_side; act_on %" PRId64 "", core_obj->id, PVC_TYPE, act_on);
 
   if (other_side->closed || other_side->read_state.shutdown) {
-    write_state.vio._cont->handleEvent(VC_EVENT_ERROR, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_ERROR, &write_state.vio);
     return;
   }
 
@@ -520,7 +514,7 @@ PluginVC::process_write_side(bool other_side_call)
     if (ntodo > 0) {
       // Notify the continuation that we are "disabling"
       //  ourselves due to to nothing to write
-      write_state.vio._cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
+      write_state.vio.cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
     }
     return;
   }
@@ -548,9 +542,9 @@ PluginVC::process_write_side(bool other_side_call)
   Debug("pvc", "[%u] %s: process_write_side; added %" PRId64 "", core_obj->id, PVC_TYPE, added);
 
   if (write_state.vio.ntodo() == 0) {
-    write_state.vio._cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &write_state.vio);
   } else {
-    write_state.vio._cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
+    write_state.vio.cont->handleEvent(VC_EVENT_WRITE_READY, &write_state.vio);
   }
 
   update_inactive_time();
@@ -631,7 +625,7 @@ PluginVC::process_read_side(bool other_side_call)
 
   if (act_on <= 0) {
     if (other_side->closed || other_side->write_state.shutdown) {
-      read_state.vio._cont->handleEvent(VC_EVENT_EOS, &read_state.vio);
+      read_state.vio.cont->handleEvent(VC_EVENT_EOS, &read_state.vio);
     }
     return;
   }
@@ -663,9 +657,9 @@ PluginVC::process_read_side(bool other_side_call)
   Debug("pvc", "[%u] %s: process_read_side; added %" PRId64 "", core_obj->id, PVC_TYPE, added);
 
   if (read_state.vio.ntodo() == 0) {
-    read_state.vio._cont->handleEvent(VC_EVENT_READ_COMPLETE, &read_state.vio);
+    read_state.vio.cont->handleEvent(VC_EVENT_READ_COMPLETE, &read_state.vio);
   } else {
-    read_state.vio._cont->handleEvent(VC_EVENT_READ_READY, &read_state.vio);
+    read_state.vio.cont->handleEvent(VC_EVENT_READ_READY, &read_state.vio);
   }
 
   update_inactive_time();
@@ -751,7 +745,7 @@ PluginVC::process_timeout(Event **e, int event_to_send)
   if (closed) {
     // already closed, ignore the timeout event
     // to avoid handle_event asserting use-after-free
-    *e = nullptr;
+    clear_event(e);
     return;
   }
 
@@ -764,8 +758,8 @@ PluginVC::process_timeout(Event **e, int event_to_send)
       }
       return;
     }
-    *e = nullptr;
-    read_state.vio._cont->handleEvent(event_to_send, &read_state.vio);
+    clear_event(e);
+    read_state.vio.cont->handleEvent(event_to_send, &read_state.vio);
   } else if (write_state.vio.op == VIO::WRITE && !write_state.shutdown && write_state.vio.ntodo() > 0) {
     MUTEX_TRY_LOCK(lock, write_state.vio.mutex, (*e)->ethread);
     if (!lock.is_locked()) {
@@ -775,13 +769,25 @@ PluginVC::process_timeout(Event **e, int event_to_send)
       }
       return;
     }
-    *e = nullptr;
-    write_state.vio._cont->handleEvent(event_to_send, &write_state.vio);
+    clear_event(e);
+    write_state.vio.cont->handleEvent(event_to_send, &write_state.vio);
   } else {
-    *e = nullptr;
+    clear_event(e);
   }
 }
 
+void
+PluginVC::clear_event(Event **e)
+{
+  if (e == nullptr || *e == nullptr)
+    return;
+  if (*e == inactive_event) {
+    inactive_event->cancel();
+    inactive_timeout_at = 0;
+  }
+  *e = nullptr;
+}
+
 void
 PluginVC::update_inactive_time()
 {
@@ -930,6 +936,18 @@ PluginVC::set_remote_addr()
   }
 }
 
+void
+PluginVC::set_remote_addr(const sockaddr * /* new_sa ATS_UNUSED */)
+{
+  return;
+}
+
+void
+PluginVC::set_mptcp_state()
+{
+  return;
+}
+
 int
 PluginVC::set_tcp_init_cwnd(int /* init_cwnd ATS_UNUSED */)
 {
@@ -1003,11 +1021,9 @@ PluginVC::set_data(int id, void *data)
 
 // PluginVCCore
 
-vint32 PluginVCCore::nextid = 0;
+int32_t PluginVCCore::nextid;
 
-PluginVCCore::~PluginVCCore()
-{
-}
+PluginVCCore::~PluginVCCore() {}
 
 PluginVCCore *
 PluginVCCore::alloc(Continuation *acceptor)
@@ -1254,9 +1270,7 @@ private:
   unsigned completions_received;
 };
 
-PVCTestDriver::PVCTestDriver() : NetTestDriver(), i(0), completions_received(0)
-{
-}
+PVCTestDriver::PVCTestDriver() : NetTestDriver(), i(0), completions_received(0) {}
 
 PVCTestDriver::~PVCTestDriver()
 {
